@@ -11,31 +11,16 @@ with 'SHARYANTO::Role::Doc::Section';
 with 'SHARYANTO::Role::I18N';
 with 'SHARYANTO::Role::I18NRinci';
 
-has url => (is=>'rw');
-has _pa => (
-    is => 'rw',
-    lazy => 1,
-    default => sub {
-        require Perinci::Access;
-        require Perinci::Access::Perl;
-        my $pa = Perinci::Access->new;
-        # slightly reduce startup overhead by avoiding to compile sah schemas
-        my $pap = Perinci::Access::Perl->new(
-            extra_wrapper_args => {
-                validate_args => 0,
-            },
-        );
-        $pa->{handlers}{pl} = $pap;
-        delete $pa->{handlers}{''};
-        $pa;
-    },
-); # store Perinci::Access object
+has meta => (is=>'rw');
+has name => (is=>'rw');
 
-our $VERSION = '0.27'; # VERSION
+our $VERSION = '0.28'; # VERSION
 
 sub BUILD {
     my ($self, $args) = @_;
 
+    $args->{meta} or die "Please specify meta";
+
     $self->{doc_sections} //= [
         'summary',
         'description',
@@ -48,32 +33,10 @@ sub BUILD {
 
 sub before_gen_doc {
     my ($self, %opts) = @_;
-
-    $self->{url} or die "Please specify url";
+    $log->tracef("=> FuncBase's before_gen_doc(opts=%s)", \%opts);
 
     # initialize hash to store [intermediate] result
     $self->{_doc_res} = {};
-
-    # let's retrieve the metadatas first, skip if already provided in _meta
-    return if $self->{_doc_meta};
-
-    $log->tracef("=> FuncBase's before_gen_doc(opts=%s)", \%opts);
-
-    my $res = $self->_pa->request(info=>$self->{url});
-    $res->[0] == 200 or die "Can't info $self->{url}: $res->[0] - $res->[1]";
-    $self->{_doc_info} = $res->[2];
-    #$log->tracef("info=%s", $self->{_doc_info});
-
-    die "url must be a function entity, not $self->{_doc_info}{type} ($self->{url})"
-        unless $self->{_doc_info}{type} eq 'function';
-
-    $res = $self->_pa->request(meta=>$self->{url});
-    if ($res->[0] == 200) {
-        $self->{_doc_meta} = $res->[2];
-        $self->{_doc_orig_meta} = $res->[3]{orig_meta};
-        #$log->tracef("meta=%s", $self->{_doc_meta});
-        #$log->tracef("orig_meta=%s", $self->{_doc_orig_meta});
-    }
 }
 
 # provide simple default implementation without any text wrapping. subclass such
@@ -97,36 +60,34 @@ sub add_doc_lines {
 sub gen_doc_section_summary {
     my ($self) = @_;
 
-    my $name;
-    my $summary = "";
-    if ($self->{_doc_meta}) {
-        $name    = $self->langprop($self->{_doc_meta}, "name");
-        $summary = $self->langprop($self->{_doc_fmeta}, "summary");
-    }
-    $name //= do {
-        my $fname = $self->{url};
-        $fname =~ s!.+/!!;
-        $fname;
-    };
+    my $meta = $self->meta;
+    my $dres = $self->{_doc_res};
 
-    $self->{_doc_res}{name}    = $name;
-    $self->{_doc_res}{summary} = $summary;
+    my $name = $self->name // $self->langprop($meta, "name") //
+        "unnamed_function";
+    my $summary = $self->langprop($meta, "summary");
+
+    $dres->{name}    = $name;
+    $dres->{summary} = $summary;
 }
 
 sub gen_doc_section_description {
     my ($self) = @_;
 
-    $self->{_doc_res}{description} = $self->{_doc_meta} ?
-        $self->langprop($self->{_doc_meta}, "description") : undef;
+    my $meta = $self->meta;
+    my $dres = $self->{_doc_res};
+
+    $dres->{description} = $self->langprop($meta, "description");
 }
 
 sub gen_doc_section_arguments {
     my ($self) = @_;
-    my $meta  = $self->{_doc_meta};
-    my $ometa = $self->{_doc_orig_meta};
+
+    my $meta = $self->meta;
+    my $dres = $self->{_doc_res};
 
     # perl term for args, whether '%args' or '@args' etc
-    my $aa = $ometa->{args_as} // $meta->{args_as} // 'hash';
+    my $aa = $meta->{args_as} // 'hash';
     my $aplt;
     if ($aa eq 'hash') {
         $aplt = '(%args)';
@@ -139,11 +100,11 @@ sub gen_doc_section_arguments {
     } else {
         die "BUG: Unknown value of args_as '$aa'";
     }
-    $self->{_doc_res}{args_plterm} = $aplt;
+    $dres->{args_plterm} = $aplt;
 
     my $args  = $meta->{args} // {};
-    $self->{_doc_res}{args} = {};
-    my $raa = $self->{_doc_res}{args};
+    $dres->{args} = {};
+    my $raa = $dres->{args};
     for my $name (keys %$args) {
         my $arg = $args->{$name};
         $arg->{default_lang} //= $meta->{default_lang};
@@ -165,23 +126,22 @@ sub gen_doc_section_arguments {
 sub gen_doc_section_result {
     my ($self) = @_;
 
-    my $meta  = $self->{_doc_meta};
-    my $ometa = $self->{_doc_orig_meta};
-    my $res   = $self->{_doc_res};
+    my $meta = $self->meta;
+    my $dres = $self->{_doc_res};
 
-    $res->{res_schema} = $meta->{result} ? $meta->{result}{schema} : undef;
-    $res->{res_schema} //= [any => {}];
-    $res->{human_res} = Perinci::ToUtil::sah2human_short($res->{res_schema});
+    $dres->{res_schema} = $meta->{result} ? $meta->{result}{schema} : undef;
+    $dres->{res_schema} //= [any => {}];
+    $dres->{human_res} = Perinci::ToUtil::sah2human_short($dres->{res_schema});
 
-    my $rn = $ometa->{result_naked} // $meta->{result_naked};
+    my $rn = $meta->{result_naked};
     if ($rn) {
-        $res->{human_ret} = $res->{human_res};
+        $dres->{human_ret} = $dres->{human_res};
     } else {
-        $res->{human_ret} = '[status, msg, result, meta]';
+        $dres->{human_ret} = '[status, msg, result, meta]';
     }
 
-    $res->{res_summary}     = $self->langprop($meta->{result}, "summary");
-    $res->{res_description} = $self->langprop($meta->{result}, "description");
+    $dres->{res_summary}     = $self->langprop($meta->{result}, "summary");
+    $dres->{res_description} = $self->langprop($meta->{result}, "description");
 }
 
 sub gen_doc_section_examples {
@@ -207,10 +167,27 @@ Perinci::Sub::To::FuncBase - Base class for Perinci::Sub::To::* function documen
 
 =head1 VERSION
 
-version 0.27
+version 0.28
 
 =for Pod::Coverage .+
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Sub-To-Text>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-To-Text>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/Public/Dist/Display.html?Name=Perinci-Sub-To-Text
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
@@ -222,11 +199,4 @@ This software is copyright (c) 2013 by Steven Haryanto.
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
-=head1 DESCRIPTION
-
-=head1 FUNCTIONS
-
-
-None are exported by default, but they are exportable.
-
 =cut
